package org.avaje.metric.filereport;

import java.io.IOException;
import java.io.Writer;
import java.text.SimpleDateFormat;
import java.util.Collection;
import java.util.Timer;
import java.util.TimerTask;
import java.util.logging.Level;
import java.util.logging.Logger;

import org.avaje.metric.Metric;
import org.avaje.metric.MetricManager;

/**
 * Writes the collected metrics to a file.
 * <p>
 * Typically you configure the frequency in seconds in which statistics are
 * collected and reported as well as a base directory where the metric files go.
 * By default the base directory will be read from a system property
 * 'metric.directory' and otherwise defaults to the current directory.
 * </p>
 */
public class FileReporter {

  private static final Logger logger = Logger.getLogger(FileReporter.class.getName());

  protected final SimpleDateFormat timeFormat = new SimpleDateFormat("HH:mm:ss");

  protected final Timer timer = new Timer("MetricFileReporter", true);

  protected final int freqInSeconds;

  protected final String baseDirectory;

  protected Writer writer;

  /**
   * Create using 60 second frequency and defined base directory.
   */
  public FileReporter(String baseDirectory) {
    this(60, baseDirectory);
  }
  
  /**
   * Create with a freqInSeconds of 60 and using the default base directory.
   */
  public FileReporter() {
    this(60, null);
  }

  /**
   * Create specifying a base directory where the metrics files should go.
   */
  public FileReporter(int freqInSeconds, String baseDirectory) {
    this.baseDirectory = getBaseDirectory(baseDirectory);
    this.freqInSeconds = freqInSeconds;
    long freqMillis = freqInSeconds * 1000;
    this.timer.scheduleAtFixedRate(new WriteTask(), freqMillis, freqMillis);
  }

  protected String getBaseDirectory(String baseDirectory) {
    if (baseDirectory == null) {
      baseDirectory = System.getProperty("metric.directory");
    }
    if (baseDirectory == null) {
      baseDirectory = ".";
    }
    return baseDirectory;
  }
  
  protected class WriteTask extends TimerTask {
    public void run() {
      try {
        writeAll();
      } catch (IOException e) {
        logger.log(Level.SEVERE, "", e);
      }
    }
  }

  protected void writeAll() throws IOException {

    FileOutput fo = new FileOutput(baseDirectory);
    try {
      writer = fo.getWriter();

      TextFileWriteVisitor visitor = new TextFileWriteVisitor(writer, freqInSeconds);
      visitor.onStartVisiting();

      Collection<Metric> allMetrics = MetricManager.getAllMetrics();
      for (Metric metric : allMetrics) {
        metric.visit(visitor);
      }

    } finally {
      fo.close();
    }
  }

}
