package org.avaje.metric.filereport;

import java.io.IOException;
import java.io.Writer;
import java.text.SimpleDateFormat;
import java.util.Date;

import org.avaje.metric.CounterStatistics;
import org.avaje.metric.LoadStatistics;
import org.avaje.metric.Metric;
import org.avaje.metric.MetricVisitor;
import org.avaje.metric.NumFormat;
import org.avaje.metric.ValueStatistics;

public class TextFileWriteVisitor implements MetricVisitor {

  protected final SimpleDateFormat timeFormat;

  protected final int collectionRateSeconds;

  protected final int decimalPlaces;

  protected final Writer writer;

  protected boolean errors;

  protected boolean emptyMetric;

  protected Metric metric;

  public TextFileWriteVisitor(Writer writer, int collectionRateSeconds) {
    this(writer, collectionRateSeconds, "HH:mm:ss");
  }

  public TextFileWriteVisitor(Writer writer, int collectionRateSeconds, String timeNowFormat) {
    this.writer = writer;
    this.timeFormat = new SimpleDateFormat(timeNowFormat);
    this.collectionRateSeconds = collectionRateSeconds;
    this.decimalPlaces = 2;
  }

  @Override
  public int getCollectionRateSeconds() {
    return collectionRateSeconds;
  }

  @Override
  public boolean isResetStatistics() {
    return true;
  }

  public void onStartVisiting() {
    try {
      writer.write(getNowString());
      writer.write(" -\n");
    } catch (IOException e) {
      throw new RuntimeException(e);
    }
  }

  @Override
  public boolean visitBegin(Metric metric, boolean emptyMetric) {
    if (emptyMetric) {
      // skip visiting/reporting this metric (implying 0 events
      // occurred since last collection)
      return false;
    }
    this.metric = metric;
    this.emptyMetric = false;
    try {
      writer.write(getNowString());
      writer.write(" ");
      writeWithPadding(metric.getName().getSimpleName(), 30);
      writer.write(" ");

      // continue visiting this metric
      return true;

    } catch (IOException e) {
      throw new RuntimeException(e);
    }
  }

  @Override
  public void visitEnd(Metric metric) {
    try {
      writer.write("\n");
    } catch (IOException e) {
      throw new RuntimeException(e);
    }
  }

  @Override
  public void visitErrorsBegin() {
    this.errors = true;
    this.emptyMetric = false;
  }

  @Override
  public void visitErrorsEnd() {
    this.errors = false;
  }
  
  @Override
  public void visit(CounterStatistics counterStatistics) {
    if (!emptyMetric) {
      try {
        writer.write("count=");
        writer.write(String.valueOf(counterStatistics.getCount()));
        writer.write(" ");
        writer.write("dur=");
        writer.write(String.valueOf(counterStatistics.getDuration()));
        writer.write(" ");
        
      } catch (IOException e) {
        throw new RuntimeException(e);
      }
    }
  }

  @Override
  public void visit(LoadStatistics loadStatistics) {
    if (!emptyMetric) {
      try {
        writer.write("count=");
        writer.write(String.valueOf(loadStatistics.getCount()));
        writer.write(" ");
        writer.write("load=");
        writer.write(String.valueOf(loadStatistics.getLoad()));
        writer.write(" ");
        writer.write("dur=");
        writer.write(String.valueOf(loadStatistics.getDuration()));
        writer.write(" ");
        
      } catch (IOException e) {
        throw new RuntimeException(e);
      }
    }
  }
  
  @Override
  public void visit(ValueStatistics summary) {
    if (!emptyMetric) {
      writeSummary("", summary);
    }
  }

  protected void writeSummary(String prefix, ValueStatistics summary) {
    try {

      long count = summary.getCount();
      writePrefix(prefix);
      writer.write("count=");
      writer.write(String.valueOf(count));
      writer.write(" ");
      if (count == 0) {
        // set emptyMetric to skip moving averages
        // which would normally follow the summary
        emptyMetric = true;
        return;
      }

      writePrefix(prefix);
      writer.write("min=");
      writer.write(NumFormat.dp(decimalPlaces, summary.getMin()));
      writer.write(" ");
      writePrefix(prefix);
      writer.write("avg=");
      writer.write(NumFormat.dp(decimalPlaces, summary.getMean()));
      writer.write(" ");
      writePrefix(prefix);
      writer.write("max=");
      writer.write(NumFormat.dp(decimalPlaces, summary.getMax()));
      writer.write(" ");
      writePrefix(prefix);
      writer.write("sum=");
      writer.write(String.valueOf(summary.getSum()));
      writer.write(" ");
      writePrefix(prefix);
      writer.write("dur=");
      writer.write(String.valueOf(summary.getDuration()));
      writer.write(" ");
      writePrefix(prefix);
      writer.write("event=");
      writer.write(NumFormat.dp(4, summary.getEventRate()));
      writer.write(" ");
      writePrefix(prefix);
      writer.write("load=");
      writer.write(NumFormat.dp(4, summary.getLoadRate()));
      writer.write(" ");

    } catch (IOException e) {
      throw new RuntimeException(e);
    }
  }

  protected void writePrefix(String prefix) {
    try {
      if (errors) {
        writer.write("err.");
      }
      if (!prefix.isEmpty()) {
        writer.write(prefix);
        // writer.write(".");
      }
    } catch (IOException e) {
      throw new RuntimeException(e);
    }
  }

  protected void writeWithPadding(String text, int padTo) throws IOException {
    writer.write(text);
    writePadding(text, padTo);
  }

  protected void writePadding(String text, int padTo) throws IOException {
    int extra = padTo - text.length();
    if (extra > 0) {
      for (int i = 0; i < extra; i++) {
        writer.write(" ");
      }
    }
  }

  protected String getNowString() {
    return timeFormat.format(new Date());
  }

}
